{
  Zur Zeit gehen nur 4 Zeilen LCD mit je 20 Zeichen
}


unit avr_lcd_2004;

{$H-}
{$O-}

interface

uses
  avr_twi;

type
  TCharMap = packed array[0..7] of  byte;

  { TAVR_LCD }
  TAVR_LCD = object
  private
    Device_Addr: Byte;

    backlightStsMask,
    displayMode, displayFunction,
    displayControl: byte;

    procedure write4Bits(Value: byte);
    procedure send(Value: byte; mode: byte);

    procedure command(Value: int16);

  public
    constructor Create(ADevice_Addr: Byte);
    destructor Destroy;

    procedure Write(s: string);

    procedure Clear;
    procedure home;

    procedure backlight(Value: boolean);

    procedure blink(Value: boolean);
    procedure cursor(Value: boolean);
    procedure display(Value: boolean);

    procedure setCursor(x, y: integer);
    procedure scrollDisplayLeft;
    procedure scrollDisplayRight;

    procedure leftToRight(Value: boolean);
    procedure Autoscroll(Value: boolean);

    procedure createChar(location: byte; const charMap: TCharMap);
  end;

implementation

const
  LCD_BLINK_ON = $01;
  LCD_CURSOR_ON = $02;
  LCD_DISPLAY_ON = $04;
  LCD_SET_DDRAM_ADDR = $80;
  LCD_CLEARDISPLAY = $01;
  LCD_RETURNHOME = $02;
  BACKLIGHT_MASK = $08;

  LCD_SETCGRAMADDR = $40;

  LCD_DISPLAYMOVE = $08;
  LCD_CURSORSHIFT = $10;
  LCD_MOVERIGHT = $04;
  LCD_MOVELEFT = $00;
  LCD_ENTRYLEFT = $02;
  LCD_ENTRYMODESET = $04;
  LCD_FUNCTIONSET = $20;
  LCD_ENTRYSHIFTINCREMENT = $01;

//  LCD_8BITMODE = $10;
  LCD_4BITMODE = $00;
  LCD_2LINE = $08;
  LCD_1LINE = $00;
//  LCD_5x10DOTS = $04;
  LCD_5x8DOTS = $00;

//  LCD_BACKLIGHT = $08;
//  LCD_NOBACKLIGHT = 00;

procedure mysleep2(t: int32);
var
  i: Int32;
begin
  for i := 0 to t do begin
    asm NOP; end;
  end;
end;

procedure Sleep;
var
  i: Integer;
begin
  for i := 1 to 4 do asm nop end;
end;


{ TAVR_LCD }
constructor TAVR_LCD.Create(ADevice_Addr: Byte);
begin
  displayControl := $08;
  displayFunction:=LCD_4BITMODE or LCD_1LINE or LCD_5x8DOTS or LCD_2LINE;

  Device_Addr := ADevice_Addr;

  backlightStsMask := BACKLIGHT_MASK;

//  TWIStart((ADSaddr0 shl 1) or TWI_Write);

//  delay_ms(50);
//  TWIwriteByte(LCD_NOBACKLIGHT,Rs);
//  mysleep2(1000);


  write4Bits($03 shl 4);
  mysleep2(4500);

  write4Bits($03 shl 4);
  mysleep2(4500);

  write4Bits($03 shl 4);
  mysleep2(150);

  write4Bits($02 shl 4);

  command(LCD_FUNCTIONSET or displayFunction);

  display(True);
  mysleep2(250);

  clear();
  mysleep2(250);

  displayMode := LCD_ENTRYLEFT;

  command(LCD_ENTRYMODESET or displayMode);
  mysleep2(250);

  home;
  mysleep2(2000);
end;

destructor TAVR_LCD.Destroy;
begin
  Clear;
  backlight(False);
end;

procedure TAVR_LCD.write4Bits(Value: byte);
const
  ENABLE_MASK: byte = $04;

  procedure Sleep;
  var
    i: Integer;
  begin
    for i := 1 to 1000 do asm nop end; // 450
  end;

begin
  TWIwriteByte(Device_Addr, Value or ENABLE_MASK or backlightStsMask);
  Sleep;
  TWIwriteByte(Device_Addr, (Value and not ENABLE_MASK) or backlightStsMask);
  Sleep;
end;

procedure TAVR_LCD.send(Value: byte; mode: byte);
var
  highnib, lownib: UInt8;
begin
  highnib := Value and $F0;
  lownib := (Value and $0F) shl 4;
//  lownib := (Value shl 4) and $F0;
  write4Bits(highnib or mode);
  write4Bits(lownib or mode);
end;

procedure TAVR_LCD.command(Value: int16);
begin
  send(Value, 0);
end;

procedure TAVR_LCD.Write(s: string);
var
  i: integer;
begin
  for i := 1 to Length(s) do begin
    send(Byte(s[i]), $01);
  end;
end;

procedure TAVR_LCD.Clear;
begin
  command(LCD_CLEARDISPLAY);
end;

procedure TAVR_LCD.home;
begin
  command(LCD_RETURNHOME);
end;

procedure TAVR_LCD.backlight(Value: boolean);
begin
  if Value then begin
    backlightStsMask := BACKLIGHT_MASK;
  end else begin
    backlightStsMask := $00;
  end;

  TWIwriteByte(Device_Addr, backlightStsMask);
end;

procedure TAVR_LCD.blink(Value: boolean);
begin
  if Value then begin
    displayControl := displayControl or LCD_BLINK_ON;
  end else begin
    displayControl := displayControl and not LCD_BLINK_ON;
  end;
  command(displayControl);
end;

procedure TAVR_LCD.cursor(Value: boolean);
begin
  if Value then begin
    displayControl := displayControl or LCD_CURSOR_ON;
  end else begin
    displayControl := displayControl and not LCD_CURSOR_ON;
  end;
  command(displayControl);
end;

procedure TAVR_LCD.display(Value: boolean);
begin
  if Value then begin
    displayControl := displayControl or LCD_DISPLAY_ON;
  end else begin
    displayControl := displayControl and not LCD_DISPLAY_ON;
  end;
  command(displayControl);
end;

procedure TAVR_LCD.setCursor(x, y: integer);
const
  ROW_ADDR: array [0..3] of byte = ($00, $40, $14, $54);
begin
//  command(LCD_SET_DDRAM_ADDR or ( ROW_ADDR[y] + x) );
  command(LCD_SET_DDRAM_ADDR + (ROW_ADDR[y mod 4] + x));
end;

procedure TAVR_LCD.scrollDisplayLeft;
begin
  command(LCD_CURSORSHIFT or LCD_DISPLAYMOVE or LCD_MOVELEFT);
end;

procedure TAVR_LCD.scrollDisplayRight;
begin
  command(LCD_CURSORSHIFT or LCD_DISPLAYMOVE or LCD_MOVERIGHT);
end;

procedure TAVR_LCD.leftToRight(Value: boolean);
begin
  if Value then begin
    displayMode := displayMode or LCD_ENTRYLEFT;
  end else begin
    displayMode := displayMode and not LCD_ENTRYLEFT;
  end;
  command(LCD_ENTRYMODESET or displayMode);
end;

procedure TAVR_LCD.Autoscroll(Value: boolean);
begin
  if Value then begin
    displayMode := displayMode or LCD_ENTRYSHIFTINCREMENT;
  end else begin
    displayMode := displayMode and not LCD_ENTRYSHIFTINCREMENT;
  end;
  command(LCD_ENTRYMODESET or displayMode);
end;

procedure TAVR_LCD.createChar(location: byte; const charMap: TCharMap);
var
  i: integer;
begin
  command(LCD_SETCGRAMADDR or ((location and $07) shl 3));
  for i := 0 to 7 do begin
    Self.Write(char(charMap[i]));
  end;
end;

end.

