{$H-}
{$O-}

{
  Autor: Michael Springwald
  Datum: 27.08.2018

  Der Am2320 ist ein Temperatur und Luftfeuchtigkeit Sensor der über I2C
  angebunden werden kann.

  Diese Variante kann zur Zeit nur mit einer Adresse umgehen.
  Ist fehlen auch noch überprüfungen z.b. wie calcCRC bei "readRegister16"
}

unit avr_sensor_am2320;

interface

uses avr_uart, avr_twi;

const
//  am2320Adress = $5C;      // 0
  AM2320_SENSOR_VERSION = $01;

  AM2320_CMD_READREG = $03;
  AM2320_REG_TEMP_H = $02;
  AM2320_REG_HUM_H = $00;

type

  { TAVR_Sensor_Am2320 }
  TAVR_Sensor_Am2320 = object
  private
  public
    Device_Addr:byte;
    Error:Boolean;

    constructor Create(ADevice_Addr: byte);
    destructor Destroy;

    function readTemp():Int16;
    function readHum():Int16;
    function readRegister16(reg:UInt8):UInt16;
    function crc16(buffer: PByte; nbytes: byte):UInt16;
  end; // TAVR_Sensor_Am2320

procedure delay_ms(time : UInt8);

implementation

procedure delay_ms(time : UInt8);
const
  fcpu = 16000000;
  fmul = 1 * fcpu div 1000000;
label
  loop1, loop2, loop3;
begin
  asm
    ldd r20, time
    loop1:
      ldi r21, fmul
      loop2:  // 1000 * fmul = 1000 * 1 * 8 = 8000 cycles / 8MHz
        ldi r22, 250
        loop3:  // 4 * 250 = 1000 cycles
          nop
          dec r22
          brne loop3
        dec r21
        brne loop2
      dec r20
      brne loop1
  end['r20','r21','r22'];
end;


function TAVR_Sensor_Am2320.readTemp(): Int16;
var
  Temp_T:Int16;
begin
  Error:=false;
  Temp_T:=readRegister16(AM2320_REG_TEMP_H);
  if Temp_T = $FFFF then begin
    Error:=true;
    exit;
  end;

  if Temp_T and $8000 <> 0 then
    result:=-int16(Temp_T and $7FFF)
  else
    result:=Temp_T;
end;

function TAVR_Sensor_Am2320.readHum(): Int16;
var
  Temp_H:int16;
begin
  Error:=false;
  Temp_H:=readRegister16(AM2320_REG_HUM_H);
  if Temp_H = $fff then begin
    Error:=true;
    exit;
  end;
  result:=Temp_H;
end;

function TAVR_Sensor_Am2320.readRegister16(reg: UInt8): UInt16;
var
  Buffer: uInt8Array;
  i:Integer;
  ret:UInt16;

  the_crc,calc_crc:UInt16;
begin
  Error:=False;

  // Aufwecken
  TWIStart((Device_Addr shl 1) or TWI_Write);
  TWIWrite($00);
  TWIStop;
  delay_ms(10);

  // Ein Kommando Senden zum "Lese Register"
  TWIStart((Device_Addr shl 1) or TWI_Write);
  TWIWrite(AM2320_CMD_READREG);
  TWIWrite(reg);
  TWIWrite(2);
  TWIStop;
  delay_ms(2);

//  Hier müsste noch eine Überprüfung vorher hin:

//  2 bytes preamble, 2 bytes data, 2 bytes CRC
//  Wire.requestFrom(_i2caddr, (uint8_t)6);
//  if (Wire.available() != 6)
//  return 0xFFFF;

  TWIStart((Device_Addr shl 1) or TWI_Read);
  for i:=0 to 5 do begin
    buffer[i]:=TWIReadACK;
  end;
  TWIStop;

  // Hier müsste noch eine calc_crc prüfung hin
  the_crc:=buffer[5];
  the_crc:=the_crc shl 8;
  the_crc:=the_crc or buffer[4];

  calc_crc:=crc16(@buffer,4);
  if the_crc <> calc_crc then begin
    result:=0;
    Error:=true;
    exit;
  end;

  ret:=buffer[2];
  ret:=ret shl 8;
  ret:=ret or buffer[3];

  result:=ret;
end;

{ TAVR_Sensor_Am2320 }
constructor TAVR_Sensor_Am2320.Create(ADevice_Addr: Byte);
begin
  Device_Addr:=ADevice_Addr;
  Error:=false;
end; // TAVR_Sensor_Am2320.Create

destructor TAVR_Sensor_Am2320.Destroy;
begin

end; // TAVR_Sensor_Am2320.Destroy

function TAVR_Sensor_Am2320.crc16(buffer: PByte; nbytes: byte): UInt16;
var
  crc: UInt16 = $FFFF;
  b: UInt8;
  i, x: byte;
begin
  for i := 0 to nbytes - 1 do begin
    b := buffer[i];
    crc := crc xor b;
    for x := 0 to 7 do begin
      if crc and $0001 <> 0 then begin
        crc := crc shr 1;
        crc := crc xor $A001;
      end else begin
        crc := crc shr 1;
      end;
    end;
  end;
  Result := crc;
end;

end.

