{
  http://wiki.freepascal.org/AVR_Embedded_Tutorial_-_I%C2%B2C,_TWI/de
}

{$H-}
unit avr_twi;

interface

uses avr_uart;

const
  CPU_Clock = 16000000; // Taktfrequenz Arduino, default 16MHz.
  I2C_Takt  = 400000;   // I²C-Takt (100KHz)
  TWI_Write = 0;
  TWI_Read  = 1;

procedure TWIInit;
function TWIStart(addr: byte):boolean;
procedure TWIStop;
procedure TWIWrite(u8data: byte);
function TWIReadACK: byte;
function TWIReadNACK: byte;
procedure TWIwriteByte(addr : Byte; b: Byte);

implementation

procedure TWIInit;
const
  TWBR_val = byte((CPU_Clock div I2C_Takt) - 16) div 2;
begin
 TWSR := 0;
 TWBR := byte(TWBR_val);
end;

function TWIStart(addr: byte): boolean;
var
  err:byte;
begin
  result:=true;
  err:=0;
  // Senden/Empfangen einleiten
  TWCR := 0;
  TWCR := (1 shl TWINT) or (1 shl TWSTA) or (1 shl TWEN);

  while ((TWCR and (1 shl TWINT)) = 0)  and (err < 255) do begin
    inc(err);
  end;

  if err >= 255 then begin
    UARTSendString('TWIStart: I2C Time Out'#13#10);
    result:=false;
    exit;
  end;

  // Adresse des Endgerätes senden
  TWDR := addr;
  TWCR := (1 shl TWINT) or (1 shl TWEN);
  while ((TWCR and (1 shl TWINT)) = 0) do begin
  end;
end;

procedure TWIStop;
begin
  TWCR := (1 shl TWINT) or (1 shl TWSTO) or (1 shl TWEN);
end;

procedure TWIWrite(u8data: byte);
var
  err:byte;
begin
  err:=0;
  TWDR := u8data;
  TWCR := (1 shl TWINT) or (1 shl TWEN);
  while ((TWCR and (1 shl TWINT)) = 0) and (err < 255) do begin
    inc(err);
  end;

  if err >= 255 then begin
    UARTSendString('TWIWrite: I2C Time Out'#13#10);
  end;
end;

procedure TWIwriteByte(addr : Byte; b: Byte);
begin
  TWIStart((addr shl 1) or TWI_Write);
  TWIWrite(b);
  TWIStop;
end;


// Lesen bis zum vorletzten Zeichen.
function TWIReadACK: byte;
var
  err:byte;
begin
  err:=0;
  TWCR := (1 shl TWINT) or (1 shl TWEN) or (1 shl TWEA);
  while ((TWCR and (1 shl TWINT)) = 0) and (err < 255) do begin
    inc(err);
  end;

  if err >= 255 then begin
    UARTSendString('TWIReadACK: I2C Time Out'#13#10);
  end;
  Result := TWDR;
end;

// Letztes Zeichen lesen.
function TWIReadNACK: byte;
var
  err:byte;
begin
  err:=0;
  TWCR := (1 shl TWINT) or (1 shl TWEN);
  while ((TWCR and (1 shl TWINT)) = 0) and (err < 255) do begin
    inc(err);
  end;
  if err >= 255 then begin
    UARTSendString('TWIReadNACK: I2C Time Out'#13#10);
  end;

  Result := TWDR;
end;

// Lesen bis zum vorletzten Zeichen.
function TWIReadACK_Error: byte;
var
  err:byte;
begin
  err := 0;
  TWCR := (1 shl TWINT) or (1 shl TWEN) or (1 shl TWEA);
  while ((TWCR and (1 shl TWINT)) = 0) and (err < 255) do begin
    Inc(err);
  end;
  if err = 255 then begin
    UARTSendString('I²C-Timeout');
    Result := 0;
  end else begin
    Result := TWDR;
  end;
end;

// Letztes Zeichen lesen.
function TWIReadNACK_Error: byte;
var
  err:byte;
begin
  err:=0;

  TWCR := (1 shl TWINT) or (1 shl TWEN);
  while ((TWCR and (1 shl TWINT)) = 0) and (err < 255) do begin
    Inc(err);
  end;
  if err = 255 then begin
    UARTSendString('I²C-Timeout');
    Result := 0;
  end else begin
    Result := TWDR;
  end;
end;

end.

