{
  http://wiki.freepascal.org/AVR_Embedded_Tutorial_-_UART/de
}

unit avr_uart;
{$H-}

interface

type
  uInt8Array=array[0..6] of UInt8;

const
  Crxlen = 16; // Puffergrösse
  CPU_Clock = 16000000; // Taktfrequenz Arduino, default 16MHz.
  Baud      = 9600;     // Baudrate
  Teiler    = CPU_Clock div (16 * Baud) - 1;

var
  RXBuf:   array[0..Crxlen - 1] of byte;
  RXread:  byte = 0;
  RXwrite: byte = 0;

procedure UARTInit;
function UARTReadChar: char;
procedure UARTSendString(s: ShortString);
procedure UARTSendIntArray(IntArray: uInt8Array);
procedure UARTSendByte(c: Byte);

implementation

procedure UARTInit;
begin
  UBRR0  := Teiler;                          // Baud
  UCSR0A := 0;                               // Normale Geschwindigkeit
  UCSR0B := (1 shl TXEN0) or (1 shl RXEN0); // Empfangen und Senden
  UCSR0C := (%011 shl UCSZ0);               // 8-Bit Übertragung, 1 Stop-Bits.
end; // UARTInit

function UARTReadChar: char;
begin
  while UCSR0A and (1 shl RXC0) = 0 do; // Warten, bis Zeichen ankommt.
  Result := char(UDR0);                 // Zeichen einlesen.
end; // UARTReadChar

procedure UARTSendChar(c: char);
begin
  while UCSR0A and (1 shl UDRE0) = 0 do; // Warten, bis letztes Zeichen gesendet.
  UDR0 := byte(c);                       // Zeichen senden.
end; // UARTSendChar

procedure UARTSendByte(c: Byte);
begin
  while UCSR0A and (1 shl UDRE0) = 0 do; // Warten, bis letztes Zeichen gesendet.
  UDR0 := c;                       // Zeichen senden.
end; // UARTSendByte

procedure UARTSendString(s: ShortString);
var
  i: integer;
begin
  for i := 1 to length(s) do begin
    UARTSendChar(s[i]);   // Zeichen einzeln senden.
  end;
end; // UARTSendString

procedure UARTSendIntArray(IntArray: uInt8Array);
var
  i: integer;
  str:string;
begin
  str:='';
  for i := 0 to length(IntArray) do begin
   str:=str+chr(IntArray[i]);
    UARTSendByte(IntArray[i]);   // Zeichen einzeln senden.
  end;
  UARTSendString(str+#13#10);
end; // UARTSendIntArray



end.

